/* Copyright rgorton 2019
 *  bugs.h: workaround for symbols in libc which are missing in PIN 3.x libc-dynamic
 *          and required by the path profiling tool
 *
 *  These include: fstat(), stat()
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <unistd.h>
#include <sys/file.h>

extern "C" {
int fstat_for_size(int fd, struct stat *statbuf)
{
    int     retCode = 0;
    OS_RETURN_CODE  OS_rc;
    USIZE   size;

    OS_rc = OS_FileSizeFD(fd, &size);
    if (OS_RETURN_CODE_NO_ERROR != OS_rc)
    {
        // not immediately obvious what the errno handler in PIN is - punt
        // for now until this is determined
        retCode = -1;
    }
    else
    {
        memset (statbuf, 0, sizeof(struct stat));
        statbuf->st_size = size;
    }
    return retCode;
}
}

int fstat(int fd, struct stat *statbuf)
{
    int     retCode = 0;
    OS_FILE_ATTRIBUTES attr = OS_FILE_ATTRIBUTES_NONE;
    OS_RETURN_CODE  OS_rc;

    OS_rc = OS_GetFDAttributes(fd, &attr);
    if (OS_RETURN_CODE_NO_ERROR != OS_rc)
    {
        // not immediately obvious what the errno handler in PIN is - punt
        // for now until this is determined
        retCode = -1;
    }
    else
    {
        memcpy(statbuf, &attr, sizeof(struct stat));
    }

    return retCode;
}

int stat(const char *path, struct stat *statbuf)
{
    int     retCode = 0;
    OS_FILE_ATTRIBUTES attr = OS_FILE_ATTRIBUTES_NONE;
    OS_RETURN_CODE  OS_rc;

    OS_rc = OS_GetFileAttributes(path, &attr);
    if (OS_RETURN_CODE_NO_ERROR != OS_rc)
    {
        // not immediately obvious what the errno handler in PIN is - punt
        // for now until this is determined
        retCode = -1;
    }
    else
    {
        memcpy(statbuf, &attr, sizeof(struct stat));
    }

    return retCode;
}
